@@ -3,13 +3,13 @@ package App::cpanminus::reporter;
 use warnings;
 use strict;
 
-our $VERSION = '0.13';
+our $VERSION = '0.14';
 
 use Carp ();
 use File::Spec     3.19;
 use File::HomeDir  0.58 ();
 use Test::Reporter 1.54;
-use CPAN::Testers::Common::Client 0.10;
+use CPAN::Testers::Common::Client 0.11;
 use CPAN::Testers::Common::Client::Config;
 use Parse::CPAN::Meta;
 use CPAN::Meta::Converter;
@@ -50,7 +50,7 @@ sub new {
       || File::Spec->catfile( $self->build_dir, 'build.log' )
   );
 
-  foreach my $option ( qw(quiet verbose force exclude only dry-run skip-history) ) {
+  foreach my $option ( qw(quiet verbose force exclude only dry-run skip-history ignore-versions) ) {
     my $method = $option;
     $method =~ s/\-/_/g;
     $self->$method( $params{$option} ) if exists $params{$option};
@@ -93,6 +93,12 @@ sub force {
   return $self->{_force};
 }
 
+sub ignore_versions {
+  my ($self, $ignore_versions) = @_;
+  $self->{_ignore_versions} = $ignore_versions if $ignore_versions;
+  return $self->{_ignore_versions};
+}
+
 sub quiet {
   my ($self, $quiet) = @_;
   if ($quiet) {
@@ -221,19 +227,27 @@ sub run {
     $self->{_cpanminus_version} = $1;
     $self->{_perl_version} = $2;
   }
+  else {
+      Carp::croak(
+          'Unable to find cpanminus/perl versions on build.log. '
+        . 'Please update App::cpanminus. If you think this is a mistake, '
+        . 'please send us a bug report with your version of App::cpanminus, '
+        . 'App::cpanminus::reporter, perl -V and your failing build.log file.'
+      );
+  }
 
   my $found = 0;
   my $parser;
 
   # we could go over 100 levels deep on the dependency track
   no warnings 'recursion';
-
   $parser = sub {
     my ($dist, $resource) = @_;
     (my $dist_vstring = $dist) =~ s/\-(\d+(?:\.\d)+)$/-v$1/ if $dist;
     my @test_output = ();
-    my $recording = 0;
-    my $str = '';
+    my $recording;
+    my $has_tests = 0;
+    my $found_na;
     my $fetched;
 
     while (<$fh>) {
@@ -245,24 +259,57 @@ sub run {
       elsif ( /^Entering (\S+)/ ) {
         my $dep = $1;
         $found = 1;
-        Carp::croak 'Parsing error. This should not happen. Please send us a report!' if $recording;
-        Carp::croak "Parsing error. Found '$dep' without fetching first." unless $resource;
-        print "entering $dep, $fetched\n" if $self->verbose;
+        Carp::croak 'Parsing error. This should not happen. Please send us a report!' if $recording && $recording eq 'test';
+        print "entering $dep, " . ($fetched || '(local)') . "\n" if $self->verbose;
         $parser->($dep, $fetched);
-        print "left $dep, $fetched\n" if $self->verbose;
+        print "left $dep, " . ($fetched || '(local)') . "\n" if $self->verbose;
         next;
       }
-      elsif ( $dist and /^Building and testing (?:$dist|$dist_vstring)/) {
+      elsif ( /^Running (?:Build|Makefile)\.PL/ ) {
+        $recording = 'configure';
+      }
+      elsif ( $dist and /^Building .*(?:$dist|$dist_vstring)/) {
         print "recording $dist\n" if $self->verbose;
-        $recording = 1;
+        $has_tests = 1 if /and testing/;
+        # if we got here, we need to flush the test output
+        # (so far filled with 'configure' output) and start
+        # recording the actual tests.
+        @test_output = ();
+        $recording = 'test';
       }
 
       push @test_output, $_ if $recording;
 
-      if ( $recording and ( /^Result: (PASS|NA|FAIL|UNKNOWN)/ or /^-> (FAIL|OK)/ ) ) {
-        my $result = $1;
-        $result = 'PASS' if $result eq 'OK';
-
+      my $result;
+      if ($recording) {
+        if (   /^Result: (PASS|NA|FAIL|UNKNOWN|NOTESTS)/
+           || ($recording eq 'test' && /^-> (FAIL|OK)/)
+        ) {
+          $result = $1;
+          if ($result eq 'FAIL' && $recording eq 'configure') {
+            $result = 'NA';
+          }
+          elsif ($result eq 'OK') {
+            $result = $has_tests ? 'PASS' : 'UNKNOWN';
+          }
+          elsif ($result eq 'NOTESTS') {
+              $result = 'UNKNOWN';
+          }
+        }
+        elsif ( $recording eq 'configure' && /^-> N\/A/ ) {
+            $found_na = 1;
+        }
+        elsif (  $recording eq 'configure'
+            # https://github.com/miyagawa/cpanminus/blob/devel/lib/App/cpanminus/script.pm#L2269
+              && ( /Configure failed for (?:$dist|$dist_vstring)/
+                || /proper Makefile.PL\/Build.PL/
+                || /configure the distribution/
+              )
+        ) {
+            $result = $found_na ? 'NA' : 'UNKNOWN';
+        }
+      }
+      if ($result) {
         my $dist_without_version = $dist;
         $dist_without_version =~ s/(\S+)-[\d.]+$/$1/;
 
@@ -270,12 +317,18 @@ sub run {
             print "No test output found for '$dist'. Skipping...\n"
                 . "To send test reports, please make sure *NOT* to pass '-v' to cpanm or your build.log will contain no output to send.\n";
         }
+        elsif (!$resource) {
+            print "Skipping report for local installation of '$dist'.\n";
+        }
         elsif ( defined $self->exclude && exists $self->exclude->{$dist_without_version} ) {
             print "Skipping $dist as it's in the 'exclude' list...\n" if $self->verbose;
         }
         elsif ( defined $self->only && !exists $self->only->{$dist_without_version} ) {
             print "Skipping $dist as it isn't in the 'only' list...\n" if $self->verbose;
         }
+        elsif ( !$self->ignore_versions && defined $self->{_perl_version} && ( $self->{_perl_version} ne $] ) ) {
+            print "Skipping $dist as its build Perl version ($self->{_perl_version}) differs from the currently running perl ($])...\n" if $self->verbose;
+        }
         else {
             my $report = $self->make_report($resource, $dist, $result, @test_output);
         }
